using System;
using System.Collections.Generic;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;
using Syncfusion.Drawing;
using Syncfusion.Styles;

namespace Advent.MxRules.Win.Controls
{
    public class MxrGridBase : GridControl
    {

        #region Constants and Enums

        protected const int SELECT_COL = 1;
        protected const int HEADER_ROW = 0;
        protected const int HEADER_COL = 0; //row header col
        protected const string SELECT_TEXT = "";
        protected const string CELLTYPE_TREE = "TreeCell";
        protected const string CELLTYPE_STATIC = "Static";
        protected const string CELLTYPE_CHECKBOX = "CheckBox";
        protected const string CELLTYPE_TEXTBOX = "TextBox";
        protected const string CELLTYPE_COMBOBOX = "ComboBox";
        protected const string CELLTYPE_COLHEADER = "Header";
        protected const string TAG_CHILD = "Child";
        protected const string TAG_SECTIONHEADER = "SectionHeader";
        protected const string TAG_NODRAGCOL = "NoDrag";  //for header only
        protected const int IGNORE_IDENTIFIER = 0;

        protected enum GridRowType : int
        {
            None = 0,           //default value, not valid
            SectionHeader = 1,  //not selectable
            GroupingParent = 2, //selectable
            GroupingChild = 3,  //not selectable
            FlatData = 4        //selectable
        }

        public enum SelectionState : int
        {
            Unchecked = 0,
            Checked = 1,
            None = 2
        }

        public enum GridImageIndex : byte
        {
            Expanded = TreeNodeState.Opened,
            Collapsed = TreeNodeState.Closed,
            RowHeader = 2,
            Empty = 3
        }

        #endregion

        #region members

        private System.Windows.Forms.ImageList imageList1;
        private System.Windows.Forms.ContextMenuStrip gridContextMenu;
        private System.Windows.Forms.ToolStripMenuItem mnuRemoveSelection;
        private System.Windows.Forms.ToolStripMenuItem mnuCopy;
 
      

        #endregion

        #region Constructor

        public MxrGridBase()
            :base()
        {
            this.InitializeMe();
            this.InitializeGrid();
        }

        #endregion

        #region Dispose

        protected override void Dispose( bool disposing )
        {
            try
            {
                base.Dispose( disposing );
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show( "Error disposing MxrGridBase: " + e.Message );
            }

        }
        #endregion

        #region Init Component

        private void InitializeMe()
        {
            Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle1 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
            Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle2 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
            Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle3 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
            Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle4 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
            this.imageList1 = new System.Windows.Forms.ImageList();
            this.gridContextMenu = new System.Windows.Forms.ContextMenuStrip( );
            this.mnuRemoveSelection = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuCopy = new System.Windows.Forms.ToolStripMenuItem();
            this.gridContextMenu.SuspendLayout();
            this.SuspendLayout();
            // 
            // imageList1
            // 
            this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
            this.imageList1.ImageSize = new System.Drawing.Size( 16, 16 );
            this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
            // 
            // this
            // 
            this.AllowDragSelectedCols = true;
            this.BackColor = System.Drawing.Color.FromArgb( ((int)(((byte)(242)))), ((int)(((byte)(247)))), ((int)(((byte)(253)))) );
            gridBaseStyle1.Name = "Row Header";
            gridBaseStyle1.StyleInfo.BaseStyle = "Header";
            gridBaseStyle1.StyleInfo.CellType = "Image";
            gridBaseStyle1.StyleInfo.Font.Bold = false;
            gridBaseStyle1.StyleInfo.Font.Facename = "Arial";
            gridBaseStyle1.StyleInfo.Font.Size = 8F;
            gridBaseStyle1.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
            gridBaseStyle1.StyleInfo.ImageFromByteArray = false;
            gridBaseStyle1.StyleInfo.ImageIndex = -1;
            gridBaseStyle1.StyleInfo.ImageList = this.imageList1;
            gridBaseStyle1.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo( System.Drawing.Color.White );
            gridBaseStyle1.StyleInfo.ReadOnly = true;
            gridBaseStyle1.StyleInfo.ShowButtons = Syncfusion.Windows.Forms.Grid.GridShowButtons.Hide;
            gridBaseStyle2.Name = "Column Header";
            gridBaseStyle2.StyleInfo.AutoSize = true;
            gridBaseStyle2.StyleInfo.BaseStyle = "Column Header";
            gridBaseStyle2.StyleInfo.Font.Bold = false;
            gridBaseStyle2.StyleInfo.Font.Facename = "Arial";
            gridBaseStyle2.StyleInfo.Font.Size = 8F;
            gridBaseStyle2.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Left;
            gridBaseStyle2.StyleInfo.ImageFromByteArray = false;
            gridBaseStyle2.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo( System.Drawing.Color.White );
            gridBaseStyle2.StyleInfo.TextColor = System.Drawing.Color.Navy;
            gridBaseStyle2.StyleInfo.WrapText = false;
            gridBaseStyle3.Name = "Standard";
            gridBaseStyle3.StyleInfo.Font.Facename = "Arial";
            gridBaseStyle3.StyleInfo.Font.Size = 8F;
            gridBaseStyle3.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo( System.Drawing.Color.FromArgb( ((int)(((byte)(242)))), ((int)(((byte)(247)))), ((int)(((byte)(253)))) ) );
            gridBaseStyle4.Name = "Header";
            gridBaseStyle4.StyleInfo.CellType = "Static";
            gridBaseStyle4.StyleInfo.Font.Bold = true;
            gridBaseStyle4.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo( System.Drawing.Color.White );
            gridBaseStyle4.StyleInfo.VerticalAlignment = Syncfusion.Windows.Forms.Grid.GridVerticalAlignment.Middle;
            this.BaseStylesMap.AddRange( new Syncfusion.Windows.Forms.Grid.GridBaseStyle[] {
            gridBaseStyle1,
            gridBaseStyle2,
            gridBaseStyle3,
            gridBaseStyle4} );
            this.ColCount = 1;
            this.ColWidthEntries.AddRange( new Syncfusion.Windows.Forms.Grid.GridColWidth[] {
            new Syncfusion.Windows.Forms.Grid.GridColWidth(0, 35)} );
            this.ControllerOptions = ((Syncfusion.Windows.Forms.Grid.GridControllerOptions)((((Syncfusion.Windows.Forms.Grid.GridControllerOptions.ClickCells
                        | Syncfusion.Windows.Forms.Grid.GridControllerOptions.SelectCells)
                        | Syncfusion.Windows.Forms.Grid.GridControllerOptions.ResizeCells)
                        | Syncfusion.Windows.Forms.Grid.GridControllerOptions.DragColumnHeader)
                        | Syncfusion.Windows.Forms.Grid.GridControllerOptions.ExcelLikeSelection));
            this.DefaultRowHeight = 16;
            this.Dock = System.Windows.Forms.DockStyle.Fill;
            this.EnterKeyBehavior = Syncfusion.Windows.Forms.Grid.GridDirectionType.None;
            this.Font = new System.Drawing.Font( "Arial", 8F );
            this.ForeColor = System.Drawing.SystemColors.ControlText;
            this.Location = new System.Drawing.Point( 0, 0 );
            this.Margin = new System.Windows.Forms.Padding( 0 );
            this.Name = "TheGrid";
            this.NumberedColHeaders = false;
            this.NumberedRowHeaders = false;
            this.Properties.BackgroundColorString = "Control";
            this.Properties.FixedLinesColorString = "ActiveCaption";
            this.Properties.GridLineColorString = "GrayText";
            this.Properties.ResizingCellsLinesColorString = "Red";
            this.Properties.RowHeaders = false;
            this.RightToLeft = System.Windows.Forms.RightToLeft.No;
            this.RowCount = 1;
            this.RowHeightEntries.AddRange( new Syncfusion.Windows.Forms.Grid.GridRowHeight[] {
            new Syncfusion.Windows.Forms.Grid.GridRowHeight(0, 16)} );
            this.SelectCellsMouseButtonsMask = ((System.Windows.Forms.MouseButtons)((System.Windows.Forms.MouseButtons.Left | System.Windows.Forms.MouseButtons.Middle)));
            this.Size = new System.Drawing.Size( 512, 76 );
            this.SmartSizeBox = false;
            this.TabIndex = 0;
            this.UseWaitCursor = true;
            // 
            // gridContextMenu
            // 
            this.gridContextMenu.Items.AddRange( new System.Windows.Forms.ToolStripItem[] {
            this.mnuRemoveSelection,
            this.mnuCopy} );
            this.gridContextMenu.Name = "gridContextMenu";
            this.gridContextMenu.Size = new System.Drawing.Size( 165, 48 );
            // 
            // mnuRemoveSelection
            // 
            //this.mnuRemoveSelection.Image = ResourceManager.GetImage( ResourceManager.MxrImages.Accept );
            this.mnuRemoveSelection.Name = "mnuRemoveSelection";
            this.mnuRemoveSelection.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.M)));
            this.mnuRemoveSelection.Size = new System.Drawing.Size( 164, 22 );
            this.mnuRemoveSelection.Text = "Remove";
            // 
            // mnuCopy
            // 
           // this.mnuCopy.Image = ResourceManager.GetImage( ResourceManager.MxrImages.Copy );
            this.mnuCopy.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.mnuCopy.Name = "mnuCopy";
            this.mnuCopy.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.C)));
            this.mnuCopy.Size = new System.Drawing.Size( 164, 22 );
            this.mnuCopy.Text = "Copy";

            this.gridContextMenu.ResumeLayout( false );
            this.ResumeLayout( false );

        }

        #endregion

        
        #region Initialize Grid

        private void InitializeGrid()
        {
            this.CellModels.Add( CELLTYPE_TREE, new TreeCellModel( this.Model ) );
            this.Model.Properties.BackgroundColor = System.Drawing.Color.FromArgb( 242, 247, 253 );
            this.Model.Options.SelectCellsMouseButtonsMask = MouseButtons.Left;
            this.Model.Options.DisplayEmptyRows = true;
            this.SetColWidth( 0, 0, 20 );
            this.BaseStylesMap["Row Header"].StyleInfo.ImageIndex = (int)GridImageIndex.RowHeader;
            this.RowCount = 0;
            
            //events
            this.ColsMoved += new GridRangeMovedEventHandler( MxrGridBase_ColsMoved );
            this.QueryAllowDragColumnHeader += new GridQueryDragColumnHeaderEventHandler( MxrGridBase_QueryAllowDragColumnHeader );
            this.CellClick += new GridCellClickEventHandler( MxrGridBase_CellClick );
            this.CheckBoxClick += new GridCellClickEventHandler( MxrGridBase_CheckBoxClick );
            this.SelectionChanging += new GridSelectionChangingEventHandler( MxrGridBase_SelectionChanging );
            this.PrepareViewStyleInfo += new GridPrepareViewStyleInfoEventHandler( MxrGridBase_PrepareViewStyleInfo );
            this.ResizingColumns += new GridResizingColumnsEventHandler( MxrGridBase_ResizingColumns );
            this.ClipboardCopy += new GridCutPasteEventHandler( MxrGridBase_ClipboardCopy );
            this.ClipboardCanCopy += new GridCutPasteEventHandler( MxrGridBase_ClipboardCanCopy );
            this.ClipboardPaste += new GridCutPasteEventHandler( MxrGridBase_ClipboardPaste );
            this.WrapCellNextControlInForm += new GridWrapCellNextControlInFormEventHandler( MxrGridBase_WrapCellNextControlInForm );
            this.MouseDown += new MouseEventHandler( MxrGridBase_MouseDown );
            
            this.mnuRemoveSelection.Click += new EventHandler( mnuRemoveSelection_Click );
            this.mnuCopy.Click += new EventHandler( mnuCopy_Click );
        }

        private void SetColWidths()
        {

            this.ColWidthEntries.AddRange( new Syncfusion.Windows.Forms.Grid.GridColWidth[] {
            new Syncfusion.Windows.Forms.Grid.GridColWidth(0, 0),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(1, 20),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(2, 200),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(3, 65),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(4, 65),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(5, 65),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(6, 65),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(7, 65),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(8, 65),
            new Syncfusion.Windows.Forms.Grid.GridColWidth(9, 65)} );

        }

        protected virtual void BuildColumns()
        {

        }

      

        #endregion

        #region Loading Data into the grid


        protected virtual void LoadData( bool activateCurrentCell )
        {
          
        }

        
     
        

        #region Event handling

        void MxrGridBase_ColsMoved( object sender, GridRangeMovedEventArgs e )
        {
           

        }

        void MxrGridBase_QueryAllowDragColumnHeader( object sender, GridQueryDragColumnHeaderEventArgs e )
        {
            
        }

        void MxrGridBase_PrepareViewStyleInfo( object sender, GridPrepareViewStyleInfoEventArgs e )
        {
           

        }

        void MxrGridBase_SelectionChanging( object sender, GridSelectionChangingEventArgs e )
        {
           
        }

        void MxrGridBase_CheckBoxClick( object sender, GridCellClickEventArgs e )
        {
            
        }

        void MxrGridBase_CellClick( object sender, GridCellClickEventArgs e )
        {
            
        }

        private void HandleCellSelection( int rowIndex, int colIndex, MouseButtons mouseButton )
        {
        }



        void MxrGridBase_ResizingColumns( object sender, GridResizingColumnsEventArgs e )
        {

        }

        void MxrGridBase_ClipboardCanCopy( object sender, GridCutPasteEventArgs e )
        {
            

        }

        void MxrGridBase_ClipboardCopy( object sender, GridCutPasteEventArgs e )
        {
            
        }

        void MxrGridBase_ClipboardPaste( object sender, GridCutPasteEventArgs e )
        {
            
        }

        void MxrGridBase_MouseDown( object sender, MouseEventArgs e )
        {
        }

        void MxrGridBase_WrapCellNextControlInForm( object sender, GridWrapCellNextControlInFormEventArgs e )
        {

        }

        void mnuCopy_Click( object sender, EventArgs e )
        {

        }

        void mnuRemoveSelection_Click( object sender, EventArgs e )
        {

        }

        #endregion

        #region Events and rows

        
        #endregion

        
        #region Exposed Events

        public delegate void MxrGridEventHandler( object sender, MxrGridRowRemovedEventArgs e );
        public delegate void MxrGridLeaveEventHandler( object sender, MxrGridLeaveEventArgs e );

        public event MxrGridEventHandler GridRowsRemoved;
        public event MxrGridLeaveEventHandler GridLeave;

        protected virtual void OnRowsRemoved( MxrGridRowRemovedEventArgs e )
        {
            if (GridRowsRemoved != null)
            {
                GridRowsRemoved( this, e );
            }

        }

        protected virtual void OnGridLeave( MxrGridLeaveEventArgs e )
        {
            if (GridLeave != null)
            {
                GridLeave( this, e );
            }

        }

        public class MxrGridRowRemovedEventArgs : EventArgs
        {
            private int m_iSectionIdentifier = 0;
            private DataSet m_dsRowsRemoved = null;

            public MxrGridRowRemovedEventArgs( int sectionIdentifier, DataSet removedRows )
            {
                m_iSectionIdentifier = sectionIdentifier;
                m_dsRowsRemoved = removedRows;
            }

            public int SectionIdentifier
            {
                get
                {
                    return m_iSectionIdentifier;
                }

            }

            public DataSet RemovedRows
            {
                get
                {
                    return m_dsRowsRemoved;
                }

            }

        }

        public class MxrGridLeaveEventArgs : EventArgs
        {
            private bool m_bForward = false;


            public MxrGridLeaveEventArgs( bool forward )
            {
                m_bForward = forward;
            }

            public bool Forward
            {
                get
                {
                    return m_bForward;
                }

            }

        }


        #endregion
        #endregion
    }

}
