using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.Serialization;

namespace Advent.MxRules.Win.Controls
{
    using Syncfusion.Windows.Forms;
	using Syncfusion.Windows.Forms.Grid;

    public enum TreeNodeState : int
    {
        Opened = 0,
        Closed = 1,
        NoChildren = 2,
    }


	//we override the Model to create the create a new cell renderer for our control
	public class TreeCellModel : GridStaticCellModel
	{
       
		
		protected TreeCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}

		public TreeCellModel(GridModel grid)
			: base(grid)
		{
				
		}
	
		//note that this method creates our new derived renderer
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new TreeCellRenderer(control, this);
		}

        
	}

	// handles drawing our indented cell
	public class TreeCellRenderer: GridStaticCellRenderer
	{
		private int _indentSize;
		private int bitmapWidth;
		private int bitmapHeight;

		public int IndentSize 
		{
			get{ return _indentSize;}
			set
			{ 
				if(value >= 0)
					_indentSize = value;
				else 
					_indentSize = 0;
			}

		}
		public TreeCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
				_indentSize = 20;
				bitmapWidth = 16;
				bitmapHeight = 16;

		}

		protected override void OnDraw(System.Drawing.Graphics g, System.Drawing.Rectangle clientRectangle, int rowIndex, int colIndex, Syncfusion.Windows.Forms.Grid.GridStyleInfo style)
		{
			if (clientRectangle.IsEmpty)
				return;
			
			int imageIndex = style.ImageIndex;
			ImageList imageList = style.ImageList;
			if (imageIndex != -1 && imageList != null && imageIndex < imageList.Images.Count)
			{
				try
				{
					Rectangle rect = GetCellBoundsCore(rowIndex, colIndex);
                    //only one level for now
					//int X = rect.X + IndentSize * Convert.ToInt32(style.Tag);
                    int X = rect.X + IndentSize * 0;
					g.DrawImage((Bitmap) imageList.Images[imageIndex], X, rect.Y);

					//now draw text past the image...
					bool drawDisabled = false;
					string displayText = String.Empty;

					X += imageList.Images[imageIndex].Width + 2;

					Rectangle textRectangle = RemoveMargins(clientRectangle, style);
					int shift = X - textRectangle.X;
					textRectangle.X = X;
					textRectangle.Width -= shift;
					if (textRectangle.IsEmpty)
						return;

					try
					{
						displayText = Model.GetFormattedOrActiveTextAt(rowIndex, colIndex, style);
					}
					catch 
					{
						displayText = style.Text;
						//style.ToolTip = ex.Message;
						drawDisabled = true;
					}

					if (style.HasError)
					{
						displayText = style.Error;
						drawDisabled = true;
					}

					if (displayText.Length > 0)
					{
						Font font = style.Font.GdipFont;
						Color textColor = Grid.Model.Properties.BlackWhite ? Color.Black : style.TextColor;
						DrawText(g, displayText, font, textRectangle, style, textColor, drawDisabled);
					}
				}
				catch (Exception e)
                {
                    System.Windows.Forms.MessageBox.Show( e.Message );
                }
			}
		}
               
		private void ClickTreeNode(int rowIndex, int colIndex, System.Windows.Forms.MouseEventArgs e)
		{
//			//process if it is a click on the current cell in column 1
//			if( (Control.ModifierKeys & Keys.Shift) != 0)//if shift move right
//			{
//				this.Grid.Model[rowIndex, colIndex].Tag = (int)this.Grid.Model[rowIndex, colIndex].Tag + 1;
//			//	this.Grid.Model[rowIndex, colIndex].ImageIndex = Form1.OpenedBMP;// force it open
//			}
//			else if( (Control.ModifierKeys & Keys.Control) != 0)//if ctrl move left
//			{
//				this.Grid.Model[rowIndex, colIndex].Tag = ((int)this.Grid.Model[rowIndex, colIndex].Tag > 0) 
//					? (int)this.Grid.Model[rowIndex, colIndex].Tag - 1 : 0;
//			}
//			else 
//			{
				Rectangle rect = GetCellBoundsCore(rowIndex, colIndex);
                //only one level 
				//int X = rect.X + IndentSize * Convert.ToInt32(this.Grid.Model[rowIndex, colIndex].Tag);
                //int X = rect.X + IndentSize * 0;
				//rect.X = X;
				rect.Width = this.bitmapWidth;
				rect.Height = this.bitmapHeight;

				if(rect.Contains(new Point(e.X, e.Y)))
				{
					if(this.Grid.Model[rowIndex, colIndex].ImageIndex == (int) TreeNodeState.Opened)
					{
						this.Grid.Model[rowIndex, colIndex].ImageIndex = (int) TreeNodeState.Closed;
						OnCollapseRow(rowIndex, colIndex, e);
					}
					else if(this.Grid.Model[rowIndex, colIndex].ImageIndex == (int) TreeNodeState.Closed)
					{
						this.Grid.Model[rowIndex, colIndex].ImageIndex = (int) TreeNodeState.Opened;
						OnExpandRow(rowIndex,colIndex,e);
					}
				}
//			}
			
			//base.OnClick(rowIndex, colIndex, e);
		}

		public virtual void OnExpandRow(int row, int col, System.Windows.Forms.MouseEventArgs e)
		{
			//Console.WriteLine("OnExpandRow " + row.ToString());
            RaiseClick(row, col, e);
		}


        public virtual void OnCollapseRow(int row, int col, System.Windows.Forms.MouseEventArgs e)
		{
			//Console.WriteLine("OnCollapseRow " + row.ToString() );
            RaiseClick(row, col, e);
        }

        
        #region events

        //public delegate void MxrTreeNodeEventHandler(object sender, int rowIndex);
      
        #endregion

        protected override void OnMouseDown( int rowIndex, int colIndex, System.Windows.Forms.MouseEventArgs e )
        {
            if (e.Button != MouseButtons.Right)
            {
                base.OnMouseDown( rowIndex, colIndex, e );
                ClickTreeNode( rowIndex, colIndex, e );
            }
        }

        protected override int OnHitTest(int rowIndex, int colIndex, MouseEventArgs e, IMouseController controller) 
		{
			if(e.Button != MouseButtons.None )
			{
				//return a cell hit so OnMouseDown will be called later
				return GridHitTestContext.Cell; 
            }
			else 
				return 0;
		}

	}

	
}
